Package: fs
Title: Cross-Platform File System Operations Based on 'libuv'
Version: 2.1.0
Authors@R: c(
    person("Jim", "Hester", role = "aut"),
    person("Hadley", "Wickham", role = "aut"),
    person("Gábor", "Csárdi", role = "aut"),
    person("Jeroen", "Ooms", , "jeroenooms@gmail.com", role = "cre"),
    person("libuv project contributors", role = "cph",
           comment = "libuv library"),
    person("Joyent, Inc. and other Node contributors", role = "cph",
           comment = "libuv library"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: A cross-platform interface to file system operations, built
    on top of the 'libuv' C library.
License: MIT + file LICENSE
URL: https://fs.r-lib.org, https://github.com/r-lib/fs
BugReports: https://github.com/r-lib/fs/issues
Depends: R (>= 4.1)
Imports: methods
Suggests: covr, crayon, knitr, pillar (>= 1.0.0), rmarkdown, spelling,
        testthat (>= 3.0.0), tibble (>= 1.1.0), vctrs (>= 0.3.0), withr
VignetteBuilder: knitr
SystemRequirements: libuv: libuv-devel (rpm) or libuv1-dev (deb).
        Alternatively to build the vendored libuv 'cmake' is required.
        GNU make.
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-23
Copyright: file COPYRIGHTS
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-18 13:56:26 UTC; jeroen
Author: Jim Hester [aut],
  Hadley Wickham [aut],
  Gábor Csárdi [aut],
  Jeroen Ooms [cre],
  libuv project contributors [cph] (libuv library),
  Joyent, Inc. and other Node contributors [cph] (libuv library),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-18 15:10:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-18 16:57:58 UTC; unix
Archs: fs.so.dSYM
