# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mkdir_ <- function(path, mode) {
    invisible(.Call(`_fs_mkdir_`, path, mode))
}

rmdir_ <- function(path) {
    invisible(.Call(`_fs_rmdir_`, path))
}

dir_map_ <- function(path, fun, all, type, recurse, fail) {
    .Call(`_fs_dir_map_`, path, fun, all, type, recurse, fail)
}

move_ <- function(path, new_path) {
    invisible(.Call(`_fs_move_`, path, new_path))
}

create_ <- function(path, mode) {
    invisible(.Call(`_fs_create_`, path, mode))
}

stat_ <- function(path, fail) {
    .Call(`_fs_stat_`, path, fail)
}

access_ <- function(path, mode) {
    .Call(`_fs_access_`, path, mode)
}

chmod_ <- function(path, mode) {
    invisible(.Call(`_fs_chmod_`, path, mode))
}

unlink_ <- function(path) {
    invisible(.Call(`_fs_unlink_`, path))
}

copyfile_ <- function(path, new_path, overwrite) {
    invisible(.Call(`_fs_copyfile_`, path, new_path, overwrite))
}

chown_ <- function(path, uid, gid) {
    invisible(.Call(`_fs_chown_`, path, uid, gid))
}

touch_ <- function(path, atime, mtime) {
    invisible(.Call(`_fs_touch_`, path, atime, mtime))
}

cleanup_ <- function() {
    invisible(.Call(`_fs_cleanup_`))
}

getmode_ <- function(mode_str, mode) {
    .Call(`_fs_getmode_`, mode_str, mode)
}

strmode_ <- function(mode) {
    .Call(`_fs_strmode_`, mode)
}

file_code_ <- function(path, mode) {
    .Call(`_fs_file_code_`, path, mode)
}

getpwnam_ <- function(name) {
    .Call(`_fs_getpwnam_`, name)
}

getgrnam_ <- function(name) {
    .Call(`_fs_getgrnam_`, name)
}

groups_ <- function() {
    .Call(`_fs_groups_`)
}

users_ <- function() {
    .Call(`_fs_users_`)
}

link_create_hard_ <- function(path, new_path) {
    invisible(.Call(`_fs_link_create_hard_`, path, new_path))
}

link_create_symbolic_ <- function(path, new_path) {
    invisible(.Call(`_fs_link_create_symbolic_`, path, new_path))
}

readlink_ <- function(path) {
    .Call(`_fs_readlink_`, path)
}

realize_ <- function(path) {
    .Call(`_fs_realize_`, path)
}

path_ <- function(paths, ext) {
    .Call(`_fs_path_`, paths, ext)
}

expand_ <- function(path, windows) {
    .Call(`_fs_expand_`, path, windows)
}

tidy_ <- function(path) {
    .Call(`_fs_tidy_`, path)
}

